@@ -3,7 +3,7 @@
 #include <stdlib.h>
 #include <selinux/selinux.h>
 
-int main(int argc, char **argv) 
+int main(int argc, char **argv)
 {
 	int rc, i;
 
@@ -15,7 +15,8 @@ int main(int argc, char **argv)
 	for (i = 2; i < argc; i++) {
 		rc = setfilecon(argv[i], argv[1]);
 		if (rc < 0) {
-			fprintf(stderr, "%s:  setfilecon(%s,%s) failed\n", argv[0], argv[i],argv[1]);
+			fprintf(stderr, "%s:  setfilecon(%s,%s) failed\n",
+				argv[0], argv[i], argv[1]);
 			exit(2);
 		}
 	}
